/*-
 * Copyright (c) 2011 The FreeBSD Foundation
 * All rights reserved.
 *
 * This software was developed by Max Khon under sponsorship from UniqueSec HB.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: ng_fst_netdev.h 10322 2011-11-04 11:55:05Z fjoe $
 */

#ifndef _NG_FARSYNC_NETDEV_H_
#define _NG_FARSYNC_NETDEV_H_

struct fst_port_info;

/**
 * Create a netgraph node and connect it to ng_iface instance
 *
 * @return 0 on success, -1 on error
 */
struct net_device *fst_netdev_create(struct fst_port_info *port, const char *name);

/**
 * Destroy a netgraph node and ng_iface instance associated with it
 */
void fst_netdev_destroy(struct net_device *dev);

/**
 * Process data frame received from the synchronous line
 */
void fst_netdev_rx(struct net_device *dev, struct mbuf *m);

/**
 * Abort receiving a data frame
 */
void fst_netdev_abort(struct net_device *dev, int event);

/**
 * Wake up transmitter
 */
void fst_netdev_wakeup_tx(struct net_device *dev);

#endif /* _NG_FARSYNC_NETDEV_H_ */
