/*
 *	FarSync WAN driver for Linux
 *
 *	Actually sync driver for X.21, V.35 and V.24 on FarSync T-series cards
 *
 *	Copyright (C) 2004 FarSite Communications Ltd.
 *	www.farsite.co.uk
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 *
 *	File:	fardsl.c
 *	Author:	K.P.Curtis	<kevin.curtis@farsite.co.uk>
 */
/*	Revision History
 *	$Log: fardsl.c,v $
 *	Revision 1.2  2009/07/10 08:23:24  kevinc
 *	Merging 1.09.04 and async branches
 *	
 *	Revision 1.1.1.1  2009/05/06 12:45:30  kevinc
 *	
 *	
 *	
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <ctype.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <linux/if.h>


#include "farsync.h"


/*	General program variables
 */
static char *progname;
static int sock;
static struct ifreq req;
static char *ifname;


static char *activation_strings[] = {
        "SDSL Power Down",
        "Send Single Pulses",
        "Initialisation",
        "Processing CL",
        "TR Analog Loop Closed",
        "TR Analog Loop Active",
        "Non TR Analog Loop Closed",
        "Non TR Analog Loop Active",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "GHS Startup",
        "GHS Transaction Auto",
        "Line Probing",
        "GHS Finished",
        "GHS Silence",
        "GHS Startup Failed",
        "GHS Transaction Failed",
        "Line Probing Failed",
        "GHS No Common Mode",
        "Time Out",
        "Idle",
        "Wait STU",
        "LP Finished",
        "Time viloation Host",
        "Invalid",
        "Invalid",
        "Invalid",
        "Wait For CR",
        "Transmit SC",
        "Transmit TC",
        "Transmit FC",
        "Invalid",
        "Invalid",
        "Invalid",
        "Transmit CR",
        "CRSR Silence",
        "Transmit SR",
        "Transmit TR",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "SR Not Detected",
        "TR Not Detected",
        "Data Exception",
        "Not Detected",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "TC Not Detected",
        "FC Not Detected",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Data",
        "Data + CRC Anomaly",
        "Data + LOSW Defect",
        "Invalid",
        "Data + Loop Atten Defect",
        "Invalid",
        "Invalid",
        "Invalid",
        "Data + SNR Margin Defect",
        "Data + ???",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Data + LOSW Failure",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Invalid",
        "Data CIE",
        "Invalid"
};

#define do_fromtable(P,T,V)			\
	if ( V >= sizeof (T) / sizeof (char *))	\
	    printf ("%sUnknown (%d)\n", P, V );	\
	else					\
	    printf ("%s%s\n", P, T[V] )

#define do_bit(V,B,S)				\
	if ( V & B ) {				\
	    if ( done_bit )			\
		printf (", ");			\
	    printf ( S );			\
	    done_bit = 1;			\
	} else /* Swallow ; */


static void
show_shdsl_state (struct fstioc_info info)
{
  static int first_time = 0;
  static int activation_status = 0;

  if (first_time == 0)
    {
      printf("Tracking the Activation Status:\n");
      first_time++;
    }
  if (activation_status != info.activationStatus)
    {
      do_fromtable("\t  Activation Status:\t", 
                   activation_strings, info.activationStatus);
      activation_status = info.activationStatus;
    }
}

static void
do_info ( void )
{
	struct fstioc_info info;

#ifdef GEN_HDLC
	union {
		fr_proto fr;
		cisco_proto cisco;
	} proto;

	bool proto_set = TRUE;
        unsigned int cmd;
#endif
	/* Zero structure before use. Good practice anyway but in this case it
	 * allows for future expansion where various fields (valid for instance)
	 * might be used to indicate an extended structure.
	 */
	memset ( &info, 0, sizeof ( info ));
	req.ifr_data = (char *)&info;
        while ( info.activationStatus < 0x40)
          {
            if ( ioctl ( sock, FSTGETCONF, &req ))
              {
                fprintf ( stderr,"%s: Problem getting port information. %s\n",
                          progname, strerror ( errno ));
                exit ( 1 );
              }
            show_shdsl_state(info);
	    usleep(500 * 1000);
          }
}

static void
usage ( void )
{
	fprintf ( stderr,"usage: %s interface -t <state>\n"
                  "Where -t set the port state.\n"
                  "use 0 for close"
                  "    1 for open\n"
                  "\n"
	, progname );
}

static int
get_config_device (char * device_name)
{
        static struct fstioc_cmd my_cmd;
        unsigned char iocinfo_version;
        char char_dev_name[32];

        /* We come in with a socket, and we may leave with a socket
         * or an fd depending on what happens
         */
        strcpy(char_dev_name, device_name);
	my_cmd.command = FSTCMD_GET_VERSION;
        my_cmd.version = 1;
	my_cmd.output_data_len=1;
        my_cmd.status = 0x55555555;
	my_cmd.data_ptr = &iocinfo_version;
	req.ifr_data = (char *)&my_cmd;
	if (ioctl(sock, FSTCMD, &req))
	  {
            /*
             * Ok, so there probably wasn't a network device
             * Try fro the char device in /dev next
             */
            close(sock);
            strcpy(char_dev_name, "/dev/");
            strcat(char_dev_name, device_name);
            if ((sock = open (char_dev_name, O_RDWR)) < 0)
              {
                /*
                 * Only once choice left now
                 */
                strcpy(char_dev_name, "/dev/tty_");
                strcat(char_dev_name, device_name);
                if ((sock = open (char_dev_name, O_RDWR)) < 0)
                  {
                    printf("Cannot find device to configure <%s>\n", 
                           device_name);
                    exit(0);
                  }
              }
	    if (ioctl(sock, FSTCMD, &req))
            {
              /*
               * One of the Char devices openend, but didn't accept the
               * command
               */
              printf("Unexpected ioctl result, errno is %d\n", errno);
            }
	  }
        if (my_cmd.status == 0xaaaaaaaa)
          {
            //printf("Using device %s for configuration\n", char_dev_name);
          }
        else
          {
            printf("Error in querying configuration device.  Status is %x\n", 
                   my_cmd.status);
            exit(0);
          }
        return sock;
}

int
main ( int argc, char *argv[] )
{
	if (( progname = strrchr ( argv[0],'/')) != NULL )
	    ++progname;
	else
	    progname = argv[0];

	if ( argc <= 1 )
	{
	    usage ();
	    exit ( 1 );
	}

	if ( strlen ( argv[1] ) >= IFNAMSIZ )
	{
	    fprintf ( stderr,"%s: Interface name too long\n", progname );
	    exit ( 1 );
	}
	strcpy ( req.ifr_name, argv[1] );
	ifname = argv[1];

	if (( sock = socket ( PF_INET, SOCK_DGRAM, IPPROTO_IP )) < 0 )
	{
	    fprintf ( stderr,"%s: Unable to create socket. %s\n", progname,
					strerror ( errno ));
	    exit ( 1 );
	}
        /*
         * Try and open a device through which to do the configuration
         * Start with the network device, then the char then the tty
         */
        sock = get_config_device(argv[1]);
        /*
         * note that when we return here, sock may actually be an fd
         */

        if (argc == 4)
          {
            int port_state;
            /*
             * There was a command line switch
             * The only one we support at the moment is -s
             * to set the state.  
             * If value following -s is 0 close port
             * If value folloiwng -s is 1 open port
             * else error
             */
            if ((strcmp(argv[2], "-t") == 0))
              {
                port_state = atoi(argv[3]);
                if((port_state == 0) || (port_state == 1))
                  {
                    printf("Setting port state to %d\n", port_state);
                    req.ifr_data = (char *) &port_state;
                    if ( ioctl ( sock, FSTSETPORT, &req ))
                      {
                        printf ("Problem setting monitor mode %s.\n"
                                    "%s\n", ifname, strerror ( errno ));
                        exit(1);
                      }
                  }
                else
                  {
                    printf("use -t 0 or -t 1\n");
                    exit (1);
                  }
                  
              }
            else
              {
                usage ();
                exit ( 1 );
              }
            /*
             * Ok to terminate with a success code now
             */
            exit (0);
          }

	/*
	 * This is the default mode to "show" dsl negotiation
	 */
	do_info();
	close ( sock );
	exit ( 0 );
}

/* End $Id: fardsl.c,v 1.2 2009/07/10 08:23:24 kevinc Exp $ */
