// fscmn.h

// Note: smcuser.h now includes this header

// Common FarSync Configuration Parameter Defines

// 03Mar08 FSCMN_TXRX_PREAMBLE added 
// 06Oct08 Correct FSCMN_MODE_DEFAULT definition
// 26Feb09 FSCMN_ASYNC_RX_XXX definitions added

#ifndef INC_FSCMN
#define INC_FSCMN

#ifdef UINT32
#define u32 UINT32
#define u16 UINT16
#define u8  UCHAR
#endif

#ifdef USSTYPES_H
#define u32 U32
#define u16 U16
#define u8  U8
#endif

// Interface defines

#define FSCMN_INTERFACE_AUTO        0
#define FSCMN_INTERFACE_V24         1
#define FSCMN_INTERFACE_X21         2
#define FSCMN_INTERFACE_V35         3
#define FSCMN_INTERFACE_X21D        4
#define FSCMN_INTERFACE_NO_CABLE    5
#define FSCMN_INTERFACE_RS530       6
#define FSCMN_INTERFACE_RS485       7

#define FSCMN_INTERFACE_DEFAULT     FSCMN_INTERFACE_V24

// Clock source defines

#define FSCMN_CLOCK_EXTERNAL        0
#define FSCMN_CLOCK_INTERNAL        1

#define FSCMN_CLOCK_DEFAULT         FSCMN_CLOCK_EXTERNAL

// Data encoding defines

#define FSCMN_ENCODING_NRZ          0x80
#define FSCMN_ENCODING_NRZI         0xa0
#define FSCMN_ENCODING_FM0          0xc0
#define FSCMN_ENCODING_FM1          0xd0
#define FSCMN_ENCODING_MANCHESTER   0xe0

#define FSCMN_ENCODING_DEFAULT      FSCMN_ENCODING_NRZ

// Data type defines

#define FSCMN_MODE_HDLC             0
#define FSCMN_MODE_TRANSPARENT      1
#define FSCMN_MODE_BISYNC           2
#define FSCMN_MODE_ASYNC            3

#define FSCMN_MODE_DEFAULT          FSCMN_MODE_HDLC

// Defines if MS or LS bit is transmitted first, LSB first for
// HDLC and MSB first for transparent is the usual setting

#define FSCMN_BIT_ORDER_LSB_FIRST   0
#define FSCMN_BIT_ORDER_MSB_FIRST   1

#define FSCMN_BIT_ORDER_DEFAULT     FSCMN_BIT_ORDER_LSB_FIRST

// Receive clock inversion defines

#define FSCMN_RXCLOCK_NORMAL        0
#define FSCMN_RXCLOCK_INVERTED      1

#define FSCMN_RXCLOCK_DEFAULT       FSCMN_RXCLOCK_NORMAL

// Start type defines

#define FSCMN_START_TX              1
#define FSCMN_START_RX              2
#define FSCMN_START_TX_AND_RX       (FSCMN_START_TX | FSCMN_START_RX)
#define FSCMN_START_DEFAULT         FSCMN_START_TX_AND_RX

// Termination defines

#define FSCMN_TERMINATION_NONE      0
#define FSCMN_TERMINATION_RESISTIVE 1

#define FSCMN_TERMINATION_DEFAULT FSCMN_TERMINATION_NONE

// Async rxFifos data format

// Char => 1-byte data (default)
// Status + Char => 1-byte status + 1-byte char
// Status + Char + Timestamp =>  1-byte status + 1-byte char + 4-byte timestamp

#define FSCMN_ASYNC_RX_CHAR             0
#define FSCMN_ASYNC_RX_STATUS_CHAR      1
#define FSCMN_ASYNC_RX_STATUS_CHAR_TIME 2

#define FSCMN_ASYNC_RX_DEFAULT FSCMN_ASYNC_RX_CHAR

// The format of the ststus byte is
// LSR format: VAL|CD |RI |CTS|BRK|FRM|PAR|OVR

typedef struct _FSCMN_TXRX_PREAMBLE
{
  u32 timestamp;    // 0x00000000 from driver to card in transmit frames
                    // 32-bit timestamp when 1st byte of receive frames
  u16 length;       // length of frame in bytes (excluding preamble)
  u8  port;         // port A or B
  u8  status;       // 0x00 from driver to card in transmit frames
                    // RSTA status byte for receive frames
} FSCMN_TXRX_PREAMBLE, *PFSCMN_TXRX_PREAMBLE;

#endif // INC_FSCMN

